import _typeof from "@babel/runtime/helpers/esm/typeof";
import _extends from "@babel/runtime/helpers/esm/extends";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _regeneratorRuntime from "@babel/runtime/helpers/esm/regeneratorRuntime";
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import { QueryBuilderDnD } from '@react-querybuilder/dnd';
import { useUpdateEffect } from 'ahooks';
import { cloneDeep, get, isEmpty } from 'lodash';
import React, { useEffect, useImperativeHandle, useMemo, useState } from 'react';
import * as ReactDnD from 'react-dnd';
import * as ReactDndHtml5Backend from 'react-dnd-html5-backend';
import QueryBuilder, { add, defaultValueProcessor, formatQuery, update } from 'react-querybuilder';
import 'react-querybuilder/dist/query-builder.css';
import PredicateItemEditor from "./PredicateItemEditor";
import AddGroupAction from "./components/AddGroupAction";
import CloneRuleAction from "./components/CloneRuleAction";
import CombinatorSelector from "./components/CombinatorSelector";
import FieldSelector from "./components/FieldSelector";
import LockRuleAction from "./components/LockRuleAction";
import OperatorSelector from "./components/OperatorSelector";
import RemoveGroupAction from "./components/RemoveGroupAction";
import RemoveRuleAction from "./components/RemoveRuleAction";
import RuleGroupExt from "./components/RuleGroupExt";
import ValueEditor from "./components/ValueEditor";
import "./styles.less";
var initialQuery = {
  combinator: 'and',
  rules: []
};
var operatorList = [{
  name: '=',
  label: '等于'
}, {
  name: '!=',
  label: '不等于'
}, {
  name: '<',
  label: '小于'
}, {
  name: '>',
  label: '大于'
}, {
  name: '<=',
  label: '小于等于'
}, {
  name: '>=',
  label: '大于等于'
}, {
  name: 'contains',
  label: '包含'
}, {
  name: 'doesNotContain',
  label: '不包含'
}];
function XQueryBuilder(props, ref) {
  var _props$verticalSize, _props$verticalSize2, _props$verticalSize3, _props$verticalSize4;
  var _useState = useState(initialQuery),
    _useState2 = _slicedToArray(_useState, 2),
    query = _useState2[0],
    setQuery = _useState2[1];
  var _useState3 = useState(),
    _useState4 = _slicedToArray(_useState3, 2),
    focusGroupId = _useState4[0],
    setFocusGroupId = _useState4[1];
  var _useState5 = useState([]),
    _useState6 = _slicedToArray(_useState5, 2),
    focusGroupPath = _useState6[0],
    setFocusGroupPath = _useState6[1];
  var _useState7 = useState(),
    _useState8 = _slicedToArray(_useState7, 2),
    editItem = _useState8[0],
    setEditItem = _useState8[1];
  var _useState9 = useState(false),
    _useState10 = _slicedToArray(_useState9, 2),
    clearEdit = _useState10[0],
    setClearEdit = _useState10[1];
  var _useState11 = useState(undefined),
    _useState12 = _slicedToArray(_useState11, 2),
    editField = _useState12[0],
    setEditField = _useState12[1];
  var _useState13 = useState('horizontal'),
    _useState14 = _slicedToArray(_useState13, 2),
    mode = _useState14[0],
    setMode = _useState14[1];

  // 监听展示方式变更
  useEffect(function () {
    setMode(props.mode || 'horizontal');
  }, [props.mode]);
  useEffect(function () {
    //初始数据加载
    if (props.jsonData && JSON.stringify(props.jsonData) !== '{}') {
      setQuery(props.jsonData);
    } else {
      setQuery({
        combinator: 'and',
        rules: []
      });
    }
    //外部插入规则
    if (props.buildItem) {
      onInsert(props.buildItem);
    }
  }, [props.jsonData, props.buildItem]);

  // 更新数据结构变更
  var refreshResult = function refreshResult() {
    var customValueProcessor = function customValueProcessor(field, operator, value) {
      return defaultValueProcessor(field, operator, value.VALUE_NAME);
    };
    if (props !== null && props !== void 0 && props.onChange) {
      var _props$onChange;
      var json = formatQuery(query, 'json');
      props === null || props === void 0 || (_props$onChange = props.onChange) === null || _props$onChange === void 0 || _props$onChange.call(props, json);
    }
  };
  // 组合选中效果
  var onGroupFocued = function onGroupFocued(id, path) {
    setFocusGroupId(id);
    setFocusGroupPath(path);
  };
  // 编辑
  var onEdit = /*#__PURE__*/function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee(info) {
      var _props$onEdit, _df$data;
      var df, x;
      return _regeneratorRuntime().wrap(function _callee$(_context) {
        while (1) switch (_context.prev = _context.next) {
          case 0:
            setEditField(info.field);
            props === null || props === void 0 || (_props$onEdit = props.onEdit) === null || _props$onEdit === void 0 || _props$onEdit.call(props, info);
            //编辑提前校验是否存在
            _context.next = 4;
            return props.getPredicateTypeDef(info.field.CLASS_TYPE_ID);
          case 4:
            df = _context.sent;
            x = df && ((_df$data = df.data) === null || _df$data === void 0 ? void 0 : _df$data.find(function (p) {
              var _field;
              return p.CLASS_ID === ((_field = info.field) === null || _field === void 0 ? void 0 : _field.CLASS_ID);
            }));
            if (!isEmpty(x)) {
              setEditItem(info);
            }
          case 7:
          case "end":
            return _context.stop();
        }
      }, _callee);
    }));
    return function onEdit(_x) {
      return _ref.apply(this, arguments);
    };
  }();
  // 结束编辑
  var onClearEdit = function onClearEdit() {
    var _props$predicateClass;
    setEditItem(undefined);
    (_props$predicateClass = props.predicateClassList) === null || _props$predicateClass === void 0 || _props$predicateClass.forEach(function (item) {
      item.disabled = false;
    });
    setClearEdit(!clearEdit);
  };
  // 外部插入
  var onInsert = function onInsert(item) {
    //新添加数据直接插入
    if (!editItem) {
      var _item$fieldDef, _item$fieldDef2;
      setQuery(add(query, {
        //@ts-ignore
        field: _objectSpread(_objectSpread({}, item.fieldDef), {}, {
          resultType: item.resultType,
          resultType_type: item.resultType_type,
          resultTypeLabel: item.resultTypeLabel,
          reviewDay: item.reviewDay,
          dateRange: item.dateRange,
          judgeResult: item.judgeResult
        }),
        operator: item.operation,
        value: item.value !== undefined ? item.value : (_item$fieldDef = item.fieldDef) !== null && _item$fieldDef !== void 0 && _item$fieldDef.INPUT_TYPE && ((_item$fieldDef2 = item.fieldDef) === null || _item$fieldDef2 === void 0 || (_item$fieldDef2 = _item$fieldDef2.INPUT_TYPE) === null || _item$fieldDef2 === void 0 ? void 0 : _item$fieldDef2.indexOf('SELECT')) > -1 ? [] : ''
      }, focusGroupPath));
    } else {
      //修改数据
      setQuery(update(update(update(query, 'field', _objectSpread(_objectSpread({}, item.fieldDef), {}, {
        resultType: item.resultType,
        resultType_type: item.resultType_type,
        resultTypeLabel: item.resultTypeLabel,
        reviewDay: item.reviewDay,
        dateRange: item.dateRange,
        judgeResult: item.judgeResult
      }), editItem.path), 'operator', item.operation, editItem.path), 'value', item.value, editItem.path));
    }
    onClearEdit();
  };

  // 联动变更触发结果数据更新通知
  useUpdateEffect(function () {
    refreshResult();
  }, [query]);

  //对外接口
  useImperativeHandle(ref, function () {
    return {
      getRule: function getRule() {
        return formatQuery(query, 'json');
      },
      setRule: function setRule(rule) {}
    };
  });

  // 获取条件列表数据
  var getPredicateTypeDef = function getPredicateTypeDef(fieldType) {
    return props.getPredicateTypeDef(fieldType);
  };
  // 获取逻辑符号列表
  var getOperatorList = function getOperatorList(classID, resultTypeID) {
    var _props$getOperatorLis;
    return ((_props$getOperatorLis = props.getOperatorList) === null || _props$getOperatorLis === void 0 ? void 0 : _props$getOperatorLis.call(props, classID, resultTypeID)) || [];
  };
  // 获取结果类型数据
  var getResultTypeList = function getResultTypeList(classID) {
    var _props$getResultTypeL;
    return ((_props$getResultTypeL = props.getResultTypeList) === null || _props$getResultTypeL === void 0 ? void 0 : _props$getResultTypeL.call(props, classID)) || [];
  };
  // 获取结果数据列表
  var getValueList = function getValueList(classID, resultTypeID, operatorName) {
    var _props$getValueList;
    return ((_props$getValueList = props.getValueList) === null || _props$getValueList === void 0 ? void 0 : _props$getValueList.call(props, classID, resultTypeID, operatorName)) || [];
  };

  // 数值修改
  var _onChangeData = function onChangeData(e, props) {
    onClearEdit();
    var list = cloneDeep(props);
    var path = list.path.join('.rules.');
    var useRule = get(query.rules, path);
    //@ts-ignore
    var operation = useRule === null || useRule === void 0 ? void 0 : useRule.operator;
    //@ts-ignore
    var val = useRule === null || useRule === void 0 ? void 0 : useRule.value;
    setQuery(update(update(update(query, 'field', e, list.path), 'operator', operation, list.path), 'value', val, list.path));
  };

  // 结果类型选择修改
  var _onChangeResultType = function onChangeResultType(e, props) {
    onClearEdit();
    var list = cloneDeep(props);
    list.value.resultType = e.key;
    list.value.resultType_type = e.resultType;
    list.value.resultTypeLabel = e.name;
    list.value.INPUT_TYPE = e.INPUT_TYPE;
    list.value.reviewDay = e.reviewDay;
    list.value.dateRange = e === null || e === void 0 ? void 0 : e.dateRange;
    list.value.judgeResult = [];
    if (e.ifSpecial) {
      setQuery(update(update(update(query, 'field', list.value, list.path), 'operator', '', list.path), 'value', '', list.path));
    } else {
      setQuery(update(update(query, 'field', list.value, list.path), 'value', '', list.path));
    }
  };

  // 结果类型数据修改
  var _onChangeJudgeResult = function onChangeJudgeResult(e, props) {
    var _query$rules$list$pat;
    onClearEdit();
    var list = cloneDeep(props);
    list.value.judgeResult = e;
    //@ts-ignore
    var val = (_query$rules$list$pat = query.rules[list.path]) === null || _query$rules$list$pat === void 0 ? void 0 : _query$rules$list$pat.value;
    setQuery(update(update(query, 'field', list.value, list.path), 'value', val, list.path));
  };
  // 条件编辑器构建
  var getQueryBuilder = useMemo(function () {
    return /*#__PURE__*/React.createElement(QueryBuilderDnD, {
      key: 1,
      dnd: _objectSpread(_objectSpread({}, ReactDnD), ReactDndHtml5Backend),
      controlElements: {
        addRuleAction: function addRuleAction() {
          return null;
        },
        //(props) => <AddRuleAction {...props}></AddRuleAction>,
        addGroupAction: function addGroupAction(p) {
          return !props.prohibitAdd && /*#__PURE__*/React.createElement(AddGroupAction, p) || null;
        },
        combinatorSelector: function combinatorSelector(props) {
          return /*#__PURE__*/React.createElement(CombinatorSelector, props);
        },
        lockGroupAction: function lockGroupAction() {
          return null;
        },
        //(props) => <LockGroupAction {...props}></LockGroupAction>,
        lockRuleAction: function lockRuleAction(props) {
          return /*#__PURE__*/React.createElement(LockRuleAction, _extends({
            editItem: editItem,
            OnEdit: onEdit
          }, props));
        },
        removeRuleAction: function removeRuleAction(p) {
          return !props.prohibitAdd && /*#__PURE__*/React.createElement(RemoveRuleAction, _extends({}, p, {
            handleOnClick: function handleOnClick(e) {
              p.handleOnClick(e);
              onClearEdit();
            }
          })) || null;
        },
        removeGroupAction: function removeGroupAction(p) {
          return !props.prohibitAdd && /*#__PURE__*/React.createElement(RemoveGroupAction, _extends({}, p, {
            handleOnClick: function handleOnClick(e) {
              p.handleOnClick(e);
              if (focusGroupId === p.ruleOrGroup.id) {
                setFocusGroupPath([]);
              }
            }
          })) || null;
        },
        fieldSelector: function fieldSelector(props) {
          return /*#__PURE__*/React.createElement(FieldSelector, _extends({}, props, {
            getPredicateTypeDef: getPredicateTypeDef,
            onChangeData: function onChangeData(e) {
              return _onChangeData(e, props);
            },
            getResultTypeList: getResultTypeList,
            onChangeResultType: function onChangeResultType(e) {
              return _onChangeResultType(e, props);
            },
            getValueList: getValueList,
            onChangeJudgeResult: function onChangeJudgeResult(e) {
              return _onChangeJudgeResult(e, props);
            }
          }));
        },
        operatorSelector: function operatorSelector(props) {
          return /*#__PURE__*/React.createElement(OperatorSelector, _extends({}, props, {
            handleOnChange: function handleOnChange(e) {
              props.handleOnChange(e);
              onClearEdit();
            },
            getOperatorList: getOperatorList
          }));
        },
        valueEditor: function valueEditor(props) {
          return /*#__PURE__*/React.createElement(ValueEditor, _extends({}, props, {
            handleOnChange: function handleOnChange(e) {
              props.handleOnChange(e);
              onClearEdit();
            },
            getResultTypeList: getResultTypeList,
            getValueList: getValueList
          }));
        },
        cloneRuleAction: function cloneRuleAction(props) {
          return /*#__PURE__*/React.createElement(CloneRuleAction, props);
        },
        cloneGroupAction: function cloneGroupAction(props) {
          return /*#__PURE__*/React.createElement(CloneRuleAction, props);
        },
        ruleGroup: function ruleGroup(props) {
          return /*#__PURE__*/React.createElement(RuleGroupExt, _extends({
            focusGroupId: focusGroupId,
            onGroupFocued: onGroupFocued
          }, props));
        }
        // rule: (props) => (
        //   <RuleExt focusGroupId={focusGroupId} onGroupFocued={onGroupFocued} {...props}></RuleExt>
        // ),
      }
    }, /*#__PURE__*/React.createElement(QueryBuilder, {
      query: query,
      operators: operatorList,
      combinators: [{
        name: 'and',
        label: '并且'
      }, {
        name: 'or',
        label: '或者'
      }],
      onAddGroup: function onAddGroup(e) {
        onClearEdit();
        return e;
      },
      enableDragAndDrop: !props.prohibitAdd,
      showLockButtons: !props.prohibitAdd,
      showCloneButtons: !props.prohibitAdd,
      controlClassnames: {
        queryBuilder: 'queryBuilder-branches'
      },
      onQueryChange: function onQueryChange(e) {
        setQuery(e);
      }
    }));
  }, [query]);
  var disable = props.disable || props.diasbled;
  return /*#__PURE__*/React.createElement("div", {
    className: "x-query-builder ".concat(disable === true && 'disable-builder'),
    style: {
      display: 'flex',
      flexDirection: mode === 'horizontal' ? 'row' : 'column'
    }
  }, /*#__PURE__*/React.createElement("div", {
    className: "xquerybuilder-builder",
    style: mode === 'vertical' ? {
      width: '100%',
      flex: "1 1 ".concat((_props$verticalSize = props.verticalSize) !== null && _props$verticalSize !== void 0 && _props$verticalSize[0] ? (_props$verticalSize2 = props.verticalSize) === null || _props$verticalSize2 === void 0 ? void 0 : _props$verticalSize2[0] : 300, "px")
    } : {}
  }, /*#__PURE__*/React.createElement("div", {
    className: "builder"
  }, props.hearderContent || props.headerContent, /*#__PURE__*/React.createElement("div", {
    className: "builderWrap ".concat(_typeof(disable) === 'object' && (disable === null || disable === void 0 ? void 0 : disable.includes('builder')) && 'disable-builder')
  }, getQueryBuilder), mode === 'horizontal' && props.extraContent && /*#__PURE__*/React.createElement("div", {
    style: _objectSpread({
      width: '100%',
      marginTop: '3px'
    }, props.extraContentStyle)
  }, props.extraContent))), /*#__PURE__*/React.createElement("div", {
    className: "xquerybuilder-field-editor ".concat(_typeof(disable) === 'object' && (disable === null || disable === void 0 ? void 0 : disable.includes('editor')) && 'disable-builder'),
    style: props.prohibitAdd ? {
      display: 'none'
    } : mode === 'vertical' ? {
      width: '100%',
      boxSizing: 'border-box',
      flex: "1 1 ".concat((_props$verticalSize3 = props.verticalSize) !== null && _props$verticalSize3 !== void 0 && _props$verticalSize3[1] ? (_props$verticalSize4 = props.verticalSize) === null || _props$verticalSize4 === void 0 ? void 0 : _props$verticalSize4[1] : 220, "px")
    } : {
      width: '',
      padding: 0
    }
  }, /*#__PURE__*/React.createElement(PredicateItemEditor, {
    getValueList: props.getValueList,
    getPredicateTypeDef: props.getPredicateTypeDef,
    onInsert: onInsert,
    editItem: editItem,
    disabled: !!disable,
    editField: editField,
    onClearEdit: onClearEdit,
    predicateClassList: props.predicateClassList,
    compact: props.compact,
    getOperatorList: props.getOperatorList,
    getResultTypeList: props.getResultTypeList,
    clearEdit: clearEdit,
    showRowNumberCol: props.showRowNumberCol,
    mode: mode,
    onChangeTab: function onChangeTab(e) {
      var _props$onChangeTab;
      setEditField(null);
      (_props$onChangeTab = props.onChangeTab) === null || _props$onChangeTab === void 0 || _props$onChangeTab.call(props, e);
    },
    onChangeSelect: props.onChangeSelect
  })), mode === 'vertical' && props.extraContent && /*#__PURE__*/React.createElement("div", {
    style: _objectSpread({
      width: '100%',
      marginTop: '3px'
    }, props.extraContentStyle)
  }, props.extraContent));
}
export default /*#__PURE__*/React.forwardRef(XQueryBuilder);