import _regeneratorRuntime from "@babel/runtime/helpers/esm/regeneratorRuntime";
import _notification from "antd/es/notification";
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
/**
 * 检查更新 Hook
 * 用于检测应用是否有新版本可用
 */
import React, { useEffect, useRef, useCallback } from 'react';
// 解析 html 中的 script 标签
// <script src="/umi.60836555.js"></script>
// 拿到 hash 值
function parseScript(html) {
  try {
    var scriptPattern = /<script[^>]*src="\/umi\.([0-9a-f]+)\.js"/i;
    var match = html.match(scriptPattern);
    if (match && match[1]) {
      return match[1];
    }
    return null;
  } catch (error) {
    console.error('Parse html error:', error);
    return null;
  }
}

// 解析 html 中的 css
// <link rel="stylesheet" href="/umi.60836555.css">
// 拿到 hash 值
function parseCss(html) {
  try {
    var cssPattern = /<link[^>]*href="\/umi\.([0-9a-f]+)\.css"/i;
    var match = html.match(cssPattern);
    if (match && match[1]) {
      return match[1];
    }
    return null;
  } catch (error) {
    console.error('Parse css error:', error);
    return null;
  }
}
export var useCheckForUpdates = function useCheckForUpdates(params) {
  var _ref = params || {},
    _ref$dev = _ref.dev,
    dev = _ref$dev === void 0 ? false : _ref$dev,
    _ref$url = _ref.url,
    url = _ref$url === void 0 ? window.location.origin : _ref$url,
    _ref$interval = _ref.interval,
    interval = _ref$interval === void 0 ? 1000 * 30 : _ref$interval,
    _ref$autoStart = _ref.autoStart,
    autoStart = _ref$autoStart === void 0 ? true : _ref$autoStart,
    onUpdateFound = _ref.onUpdateFound,
    onError = _ref.onError;

  // 当前页面的 hash，script 和 css 拼接
  var currentHashRef = useRef(null);
  // 定时器
  var timerRef = useRef(null);
  // 是否正在检查
  var isRunningRef = useRef(false);

  // 检查更新
  var checkForUpdates = useCallback( /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/_regeneratorRuntime().mark(function _callee() {
    var response, data, scriptHash, cssHash, hash, current, returnValue, errorObj;
    return _regeneratorRuntime().wrap(function _callee$(_context) {
      while (1) switch (_context.prev = _context.next) {
        case 0:
          _context.prev = 0;
          _context.next = 3;
          return fetch(url + '?t=' + Date.now(), {
            method: 'GET',
            headers: {
              'Cache-Control': 'no-cache',
              Pragma: 'no-cache'
            }
          });
        case 3:
          response = _context.sent;
          if (response.ok) {
            _context.next = 6;
            break;
          }
          throw new Error("HTTP ".concat(response.status, ": ").concat(response.statusText));
        case 6:
          _context.next = 8;
          return response.text();
        case 8:
          data = _context.sent;
          scriptHash = parseScript(data);
          cssHash = parseCss(data);
          hash = scriptHash && cssHash ? "".concat(scriptHash).concat(cssHash) : null;
          if (!hash) {
            _context.next = 23;
            break;
          }
          // 检查是否有更新
          current = currentHashRef.current;
          if (!(current && current !== hash)) {
            _context.next = 22;
            break;
          }
          isRunningRef.current = false;
          clearInterval(timerRef.current);
          timerRef.current = null;
          returnValue = onUpdateFound === null || onUpdateFound === void 0 ? void 0 : onUpdateFound(hash, current);
          if (!(returnValue === false)) {
            _context.next = 21;
            break;
          }
          return _context.abrupt("return");
        case 21:
          _notification.info({
            key: 'update-found',
            message: '发现新版本',
            description: /*#__PURE__*/React.createElement("p", {
              style: {
                fontSize: 13
              }
            }, "\u8BF7\u53CA\u65F6\u5237\u65B0\u9875\u9762\uFF0C\u4EE5\u83B7\u53D6\u6700\u65B0\u529F\u80FD", /*#__PURE__*/React.createElement("a", {
              href: "javascript:void(0)",
              style: {
                marginLeft: 4,
                color: '#1890ff',
                cursor: 'pointer'
              },
              onClick: function onClick() {
                return window.location.replace(window.location.href);
              }
            }, "\u7ACB\u5373\u5237\u65B0")),
            duration: 0
          });
        case 22:
          // 更新当前hash（如果还没有设置）
          if (!currentHashRef.current) {
            currentHashRef.current = hash;
          }
        case 23:
          _context.next = 30;
          break;
        case 25:
          _context.prev = 25;
          _context.t0 = _context["catch"](0);
          errorObj = _context.t0 instanceof Error ? _context.t0 : new Error(String(_context.t0));
          onError === null || onError === void 0 || onError(errorObj);
          console.error('Check for updates error:', errorObj);
        case 30:
        case "end":
          return _context.stop();
      }
    }, _callee, null, [[0, 25]]);
  })), [url]);

  // 开始检查
  var startCheck = useCallback(function () {
    if (dev || isRunningRef.current) return;
    isRunningRef.current = true;
    checkForUpdates();
    timerRef.current = setInterval(function () {
      checkForUpdates();
    }, interval);
  }, [dev, checkForUpdates, interval]);

  // 停止检查
  var stopCheck = useCallback(function () {
    isRunningRef.current = false;
    if (timerRef.current) {
      clearInterval(timerRef.current);
      timerRef.current = null;
    }
  }, []);

  // 初始化
  useEffect(function () {
    if (dev) {
      return;
    }
    if (autoStart) {
      startCheck();
    }
    return function () {
      stopCheck();
    };
  }, [dev, autoStart, startCheck, stopCheck]);
  return {
    startCheck: startCheck,
    stopCheck: stopCheck
  };
};